@@ -130,11 +130,11 @@ import pymbolic.mapper.stringifier
 import pymbolic.mapper.dependency
 import pymbolic.mapper.substitutor
 import pymbolic.mapper.differentiator
-import pymbolic.mapper.expander
+import pymbolic.mapper.distributor
 import pymbolic.mapper.flattener
 import pymbolic.primitives
 
-from pymbolic.polynomial import Polynomial
+from pymbolic.polynomial import Polynomial  # noqa
 
 var = pymbolic.primitives.Variable
 variables = pymbolic.primitives.variables
@@ -146,11 +146,14 @@ linear_combination = pymbolic.primitives.linear_combination
 cse = pymbolic.primitives.make_common_subexpression
 make_sym_vector = pymbolic.primitives.make_sym_vector
 
+disable_subscript_by_getitem = pymbolic.primitives.disable_subscript_by_getitem
+
 parse = pymbolic.parser.parse
 evaluate = pymbolic.mapper.evaluator.evaluate
 evaluate_kw = pymbolic.mapper.evaluator.evaluate_kw
 compile = pymbolic.compiler.compile
 substitute = pymbolic.mapper.substitutor.substitute
 diff = differentiate = pymbolic.mapper.differentiator.differentiate
-expand = pymbolic.mapper.expander.expand
+expand = pymbolic.mapper.distributor.distribute
+distribute = pymbolic.mapper.distributor.distribute
 flatten = pymbolic.mapper.flattener.flatten
