@@ -24,6 +24,56 @@ THE SOFTWARE.
 
 import pymbolic.primitives as primitives
 
+__doc__ = """
+Basic dispatch
+--------------
+
+.. autoclass:: Mapper
+
+    .. automethod:: __call__
+
+    .. method:: rec(expr, *args, **kwargs)
+
+        Identical to :meth:`__call__`, but intended for use in recursive dispatch
+        in mapper methods.
+
+    .. automethod:: handle_unsupported_expression
+
+    .. rubric:: Handling objects that don't declare mapper methods
+
+    In particular, this includes many non-subclasses of
+    :class:`pymbolic.primitives.Expression`.
+
+    .. automethod:: map_foreign
+
+    These are abstract methods for foreign objects that should be overridden
+    in subclasses:
+
+    .. method:: map_constant(expr, *args, **kwargs)
+
+        Mapper method for constants.
+        See :func:`pymbolic.primitives.register_constant_class`.
+
+    .. method:: map_list(expr, *args, **kwargs)
+
+    .. method:: map_tuple(expr, *args, **kwargs)
+
+    .. method:: map_numpy_array(expr, *args, **kwargs)
+
+Base classes for new mappers
+----------------------------
+
+.. autoclass:: CombineMapper
+
+.. autoclass:: Collector
+
+.. autoclass:: IdentityMapper
+
+.. autoclass:: WalkMapper
+
+.. autoclass:: CSECachingMapperMixin
+"""
+
 
 try:
     import numpy
@@ -117,7 +167,7 @@ class Mapper(object):
         else:
             raise ValueError(
                     "%s encountered invalid foreign object: %s" % (
-                    self.__class__, repr(expr)))
+                        self.__class__, repr(expr)))
 
 # }}}
 
@@ -152,6 +202,15 @@ class CombineMapper(RecursiveMapper):
                     self.rec(child, *args) for child in expr.parameters)
                 )
 
+    def map_call_with_kwargs(self, expr, *args):
+        return self.combine(
+                (self.rec(expr.function, *args),)
+                + tuple(
+                    self.rec(child, *args) for child in expr.parameters)
+                + tuple(
+                    self.rec(child, *args) for child in expr.kw_parameters.values())
+                )
+
     def map_subscript(self, expr, *args):
         return self.combine(
                 [self.rec(expr.aggregate, *args),
@@ -208,6 +267,9 @@ class CombineMapper(RecursiveMapper):
             self.rec(expr.left, *args),
             self.rec(expr.right, *args)))
 
+    map_max = map_sum
+    map_min = map_sum
+
     def map_list(self, expr, *args):
         return self.combine(self.rec(child, *args) for child in expr)
 
@@ -237,6 +299,31 @@ class CombineMapper(RecursiveMapper):
 # }}}
 
 
+# {{{ collector
+
+class Collector(CombineMapper):
+    """A subclass of :class:`CombineMapper` for the common purpose of
+    collecting data derived from an expression in a set that gets 'unioned'
+    across children at each non-leaf node in the expression tree.
+
+    By default, nothing is collected. All leaves return empty sets.
+
+    .. versionadded:: 2014.3
+    """
+
+    def combine(self, values):
+        import operator
+        return reduce(operator.or_, values, set())
+
+    def map_constant(self, expr):
+        return set()
+
+    map_variable = map_constant
+    map_function_symbol = map_constant
+
+# }}}
+
+
 # {{{ identity mapper
 
 class IdentityMapper(Mapper):
@@ -258,18 +345,28 @@ class IdentityMapper(Mapper):
         return expr
 
     def map_call(self, expr, *args):
-        return expr.__class__(
+        return type(expr)(
                 self.rec(expr.function, *args),
                 tuple(self.rec(child, *args)
                     for child in expr.parameters))
 
+    def map_call_with_kwargs(self, expr, *args):
+        return type(expr)(
+                self.rec(expr.function, *args),
+                tuple(self.rec(child, *args)
+                    for child in expr.parameters),
+                dict(
+                    (key, self.rec(val, *args))
+                    for key, val in expr.kw_parameters.iteritems())
+                    )
+
     def map_subscript(self, expr, *args):
-        return expr.__class__(
+        return type(expr)(
                 self.rec(expr.aggregate, *args),
                 self.rec(expr.index, *args))
 
     def map_lookup(self, expr, *args):
-        return expr.__class__(
+        return type(expr)(
                 self.rec(expr.aggregate, *args),
                 expr.name)
 
@@ -418,6 +515,17 @@ class WalkMapper(RecursiveMapper):
         for child in expr.parameters:
             self.rec(child, *args)
 
+    def map_call_with_kwargs(self, expr, *args):
+        if not self.visit(expr, *args):
+            return
+
+        self.rec(expr.function, *args)
+        for child in expr.parameters:
+            self.rec(child, *args)
+
+        for child in expr.kw_parameters.values():
+            self.rec(child, *args)
+
     def map_subscript(self, expr, *args):
         if not self.visit(expr, *args):
             return
@@ -482,6 +590,13 @@ class WalkMapper(RecursiveMapper):
         for i in indices_in_shape(expr.shape):
             self.rec(expr[i], *args)
 
+    def map_multivector(self, expr, *args):
+        if not self.visit(expr, *args):
+            return
+
+        for bits, coeff in expr.data.iteritems():
+            self.rec(coeff)
+
     def map_common_subexpression(self, expr, *args, **kwargs):
         if not self.visit(expr, *args):
             return
